#pragma once

#include "BaseBBS.h"

#include <boost/shared_ptr.hpp>

#include <boost/thread.hpp>
#include <boost/thread/recursive_mutex.hpp>

namespace bbs {


class BBSManager {
private:
	struct Mutex {
		boost::recursive_mutex bbs;
		boost::recursive_mutex value;
	};
	Mutex mutex_;

	boost::shared_ptr<BaseBBS> bbs_;

	boost::optional<BBSInfo> info_;
	boost::optional<std::vector<ThreadInfo>> subject_;
	boost::optional<std::vector<ResInfo>> dat_;
	boost::optional<int> pos_;

public:
	BBSManager();
	virtual ~BBSManager();

	operator bool();

	bool init(const std::wstring& url, const std::wstring& shitaraba, const std::wstring& userAgent, int timeout);
	void uninit();

	bool change(const std::wstring& key);

	int loadSetting(const std::wstring& userAgent, int timeout);
	int loadSubject(const std::wstring& userAgent, int timeout);
	int loadDat(const std::wstring& userAgent, int timeout);

	bool getSubject(std::vector<ThreadInfo>& data);
	bool getDat(std::vector<ResInfo>& data, int diff = 0, int len = -1);
	bool getPos(int& pos);

	int post(const std::wstring& userAgent, int timeout, const std::wstring& name, const std::wstring& mail, const std::wstring& message,
		int& code, std::wstring& error);

public:
	bool getURL(std::wstring& url);
	bool getBoardURL(std::wstring& url);
	bool getThreadURL(std::wstring& url);
	bool getBoardTitle(std::wstring& title);
	bool getThreadTitle(std::wstring& title);	
	bool getNoName(std::wstring& noname);	
	bool getCount(int& count);
	bool getKey(std::wstring& key);
	bool hasKey();
	bool getStop(int& stop);
	bool isStop();

private:
	bool match(const std::wstring& str, const std::wstring& rx);
};


}; // namespace bbs